###############################################################
#
#   SuPy - Observer base classes
#
###############################################################

def _stub(*args):
	pass

class AppObserver(object):

	__rubymodules__ = ['PyAppObserver']
	
	onNewModel = _stub
	onOpenModel = _stub
	onQuit = _stub
	

class DefinitionsObserver(object):

	__rubymodules__ = ['PyDefinitionsObserver']
	
	onComponentAdded = _stub
	onComponentPropertiesChanged = _stub
	onComponentRemoved = _stub
	onComponentTypeChanged = _stub
	

class DefinitionObserver(object):

	__rubymodules__ = ['PyDefinitionObserver']
	
	onComponentInstanceAdded = _stub
	onComponentInstanceRemoved = _stub
	

class EntitiesObserver(object):

	__rubymodules__ = ['PyEntitiesObserver']
	
	onElementAdded = _stub
	onElementRemoved = _stub
	onContentsModified = _stub
	

class EntityObserver(object):

	__rubymodules__ = ['PyEntityObserver']
	
	onEraseEntity = _stub
	onChangeEntity = _stub
	

class InstanceObserver(object):

	__rubymodules__ = ['PyInstanceObserver']

	onOpen = _stub
	onClose = _stub
	

class LayersObserver(object):

	__rubymodules__ = ['PyLayersObserver']
	
	onCurrentLayerAdded = _stub
	onLayerAdded = _stub
	onLayerRemoved = _stub
	onRemoveAllLayers = _stub

class MaterialsObserver(object):

	__rubymodules__ = ['PyMaterialsObserver']
	
	onMaterialAdd = _stub
	onMaterialChange = _stub
	onMaterialRemove = _stub
	onMaterialRefChange = _stub
	onMaterialSetCurrent = _stub
	onMaterialUndoRedo = _stub
	onMaterialRemoveAll = _stub
	

class ModelObserver(object):

	__rubymodules__ = ['PyModelObserver']
	
	onDeleteModel = _stub
	onEraseAll = _stub
	onTransactionEnd = _stub
	onTransactionStart = _stub
	

class OptionsProviderObserver(object):

	__rubymodules__ = ['PyOptionsProviderObserver']

	onOptionsProviderChanged = _stub
	

class PagesObserver(EntitiesObserver):

	__rubymodules__ = ['PyPagesObserver']
	
	onElementAdded = _stub
	onElementRemoved = _stub
	onContentsModified = _stub
	

#class RenderingOptionsObserver(object):
#
#	__rubymodules__ = ['PyRenderingOptionsObserver']
#
#   Ruby docs incomplete

class SelectionObserver(object):

	__rubymodules__ = ['PySelectionObserver']
	
	onSelectionAdded = _stub
	onSelectionBulkChange = _stub
	onSelectionCleared = _stub
	onSelectionRemoved = _stub
	

class ShadowInfoObserver(object):

	__rubymodules__ = ['PyShadowInfoObserver']
	
	onShadowInfoChanged = _stub
	

class ToolsObserver(object):

	__rubymodules__ = ['PyToolsObserver']
	
	onActiveToolChanged = _stub
	onToolStateChanged = _stub
	

class ViewObserver(object):

	__rubymodules__ = ['PyViewObserver']
	
	onViewChanged = _stub

#-------------------------------------------------------------

class _MainAppObserver(AppObserver):

	def onNewModel(self, model):
		self.delegate('onNewModel', model)
	
	def onOpenModel(self, model):
		self.delegate('onOpenModel', model)
	
	def onQuit(self):
		self.delegate('onQuit')

	def delegate(self, name, *args):
		for observer in app_observers.itervalues():
			method = getattr(observer, name, None)
			if method:
				method(*args)

#-------------------------------------------------------------

app_observers = {}
keep('app_observers')

def add_app_observer(observer, name = None):
	if not name:
		name = get_app_observer_name(observer)
	#print "add_app_observer:", name ###
	app_observers[name] = observer

def remove_app_observer(arg):
	if not isinstance(name, basestring):
		arg = get_app_observer_name(arg)
	try:
		del app_observers[name]
	except KeyError:
		pass

def get_app_observer_name(observer):
	cls = observer.__class__
	return "%s.%s" % (cls.__module__, cls.__name__)

def init_main_app_observer():
	#Sketchup.send_action("showRubyPanel:")
	#print "observer: Installing main app observer" ###
	obs = _MainAppObserver()
	Sketchup.add_observer(obs)
	return obs

keep_init('main_app_observer', init_main_app_observer)
