####################################################################
#
#   SuPy - Create release files
#
####################################################################

import os, sys, zipfile

verbose = False
version = None

version_file = "VERSION.txt"
arc_version_file = os.path.join("plugins", "supy", version_file)

loose_files = ["COPYING.txt", "README.txt"]

boring_names = ["build"]

boring_exts = [".o", ".pyc", ".pyo"]

zip = None
zip_root = None

def nonhidden_name(name):
	return not name.startswith(".")

def interesting_file(path):
	name = os.path.basename(path)
	if not nonhidden_name(name):
		return False
	if name in boring_names:
		return False
	for ext in boring_exts:
		if path.endswith(ext):
			return False
	if os.path.islink(path):
		return False
	return True

def update_version_file():
	f = open(version_file, "w")
	f.write("%s\n" % version)
	f.close()

def create_zip_file(name):
	global zip, zip_root
	filename = name + ".zip"
	print "Creating", filename
	path = os.path.join(os.pardir, filename)
	zip_root = name
	zip = zipfile.ZipFile(path, "w", zipfile.ZIP_DEFLATED)

def close_zip_file():
	zip.close()
def add_file(path, arcpath = None):
	if not arcpath:
		arcpath = path
	if verbose:
		if arcpath <> path:
			print "Adding", path, "as", arcpath
		else:
			print "Adding", path
	zip.write(path, os.path.join(zip_root, arcpath))

def add_loose_files():
	for file in loose_files:
		add_file(file)

def add_binary(target):
	target_dir = os.path.join("build", target)
	for name in os.listdir(target_dir):
		if interesting_file(name):
			file = os.path.join(target_dir, name)
			arcname = os.path.join("plugins", name)
			add_file(file, arcname)

def add_directory(root, arcroot = None):
	if not arcroot:
		arcroot = root
	for name in os.listdir(root):
		path = os.path.join(root, name)
		if interesting_file(path):
			arcpath = os.path.join(arcroot, name)
			if os.path.isdir(path):
				add_directory(path, arcpath)
			else:
				add_file(path, arcpath)

def find_targets(build_dir):
	for target in os.listdir(build_dir):
		if nonhidden_name(target):
			yield target

def create_binary_dists():
	build_dir = "build"
	for target in find_targets(build_dir):
		release_name = "SuPy-%s-%s" % (version, target)
		create_zip_file(release_name)
		add_loose_files()
		add_file(version_file, arc_version_file)
		add_binary(target)
		add_directory("doc")
		add_file("plugins/python/example.py", "doc/example.py")
		add_directory("plugins")
		close_zip_file()

def create_source_dist():
	release_name = "SuPy-%s-source" % version
	create_zip_file(release_name)
	add_directory(os.curdir)
	close_zip_file()

def main():
	global verbose, version
	args = sys.argv[1:]
	if args and args[0] == "-v":
		verbose = True
		args.pop(0)
	if len(args) < 1:
		print "Usage: release.py [-v] version"
		sys.exit(1)
	version = args[0]
	update_version_file()
	create_binary_dists()
	create_source_dist()

main()
